package com.xiaoniu.subsystem.system.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.IService;
import com.xiaoniu.api.vo.ApiLoginUserVO;
import com.xiaoniu.subsystem.system.domain.SysUser;
import com.xiaoniu.subsystem.system.domain.dto.AddSysUserDTO;
import com.xiaoniu.subsystem.system.domain.dto.EditSysUserDTO;
import com.xiaoniu.subsystem.system.domain.dto.UserDetailVO;
import com.xiaoniu.subsystem.system.domain.dto.UserListQueryDTO;
import com.xiaoniu.subsystem.system.domain.vo.UserListVO;
import jakarta.validation.Valid;

import java.util.List;

/**
 * 用户信息表 服务类
 *
 * @author sst
 * @since 2024-11-26
 */
public interface ISysUserService extends IService<SysUser> {


    /**
     * 获取用户列表
     *
     * @param dto dto
     * @return IPage<UserListVO>
     */
    IPage<UserListVO> selectUserList(IPage<UserListVO> page, UserListQueryDTO dto);

    /**
     * 用户详情
     *
     * @param userId 用户id
     * @return UserDetailVO
     */
    UserDetailVO detail(Long userId);

    /**
     * 删除用户
     *
     * @param ids id集合
     * @return
     */
    boolean del(List<Long> ids);

    /**
     * 根据条件分页查询已分配用户角色列表
     *
     * @param user 用户信息
     * @return 用户信息集合信息
     */
    public List<SysUser> selectAllocatedList(SysUser user);

    /**
     * 根据条件分页查询未分配用户角色列表
     *
     * @param user 用户信息
     * @return 用户信息集合信息
     */
    public List<SysUser> selectUnallocatedList(SysUser user);

    /**
     * 通过用户名查询用户
     *
     * @param userName 用户名
     * @return 用户对象信息
     */
    public ApiLoginUserVO selectUserByUserName(String userName);

    /**
     * 通过用户ID查询用户
     *
     * @param userId 用户ID
     * @return 用户对象信息
     */
    public SysUser selectUserById(Long userId);

    /**
     * 根据用户ID查询用户所属角色组
     *
     * @param userName 用户名
     * @return 结果
     */
    public String selectUserRoleGroup(String userName);

    /**
     * 修改
     *
     * @param dto
     * @return
     */
    boolean editUser(@Valid EditSysUserDTO dto);

    /**
     * 添加
     *
     * @param dto
     * @return
     */
    boolean addUser(@Valid AddSysUserDTO dto);


}
